//
//  ContentView.swift
//  GraphObjects
//
//  Created by Joe Gregg on 10/21/24.
//

import SwiftUI

struct ContentView: View {
    @StateObject var graph : GraphModel = GraphModel()
    @State var dragLine : Path?
    @State var drawingEdge : Bool = false
    
    var gesture :  some Gesture {
        DragGesture()
            .onChanged( { value in
                if !graph.started {
                    graph.startDrag(value.startLocation)
                } else {
                    if !drawingEdge {
                        graph.continueDrag(value.translation)
                    } 
                    dragLine = Path() { path in
                        path.move(to: value.startLocation)
                        path.addLine(to: value.location)
                    }
                }
            })
            .onEnded( { value in
                if drawingEdge {
                    graph.endDraw(value.location)
                } else {
                    graph.endDrag()
                }
                dragLine = nil
            })
    }
    
    var body: some View {
        VStack {
            Toggle("Draw Edges", isOn: $drawingEdge)
            Canvas { context, size in
                for e in graph.edges {
                    e.drawIn(context)
                }
                for v in graph.vertices {
                    v.drawIn(context)
                }
                if let dl = dragLine {
                    if drawingEdge {
                        context.stroke(dl,with:.color(.black),lineWidth:1)
                    }
                }
            }
            .gesture(
                gesture
            )
            .frame(width:200,height:200)
            .padding()
            Button("Make New Vertex") {
                graph.makeVertex(CGPoint(x:100,y:100))
            }
        }
    }
}

#Preview {
    ContentView()
}
