//
//  GraphModel.swift
//  GraphObjects
//
//  Created by Joe Gregg on 10/21/24.
//

import Foundation
import SwiftUI

class GraphModel : ObservableObject {
    @Published var vertices : [Vertex]
    @Published var edges : [Edge]
    @Published var dragging : Vertex?
    @Published var started : Bool = false
    
    init() {
        vertices = []
        let x1 = CGFloat(25.0)
        let x2 = CGFloat(75.0)
        let v1 = Vertex(center: CGPoint(x:x1,y:x1))
        let v2 = Vertex(center: CGPoint(x:x2,y:x2))
        vertices = [v1,v2]
        let edge = Edge(origin: v1, dest: v2)
        edges = [edge]
    }
    
    func makeVertex(_ center : CGPoint) {
        let v = Vertex(center: center)
        vertices.append(v)
    }
    
    func startDrag(_ pt : CGPoint) {
        started = true
        for v in vertices {
            if v.containsPoint(pt) {
                dragging = v
            }
        }
    }
    
    func continueDrag(_ offset : CGSize) {
        if let s = dragging {
            s.setOffset(offset)
        }
    }
    
    func endDrag() {
        if let s = dragging {
            s.endDrag()
        }
        dragging = nil
        started = false
    }
    
    func endDraw(_ pt : CGPoint) {
        for v in vertices {
            if v.containsPoint(pt) && (v !== dragging) {
                let newEdge = Edge(origin:dragging!,dest:v)
                edges.append(newEdge)
                break
            }
        }
    }
}
