//
//  GraphObjects.swift
//  GraphObjects
//
//  Created by Joe Gregg on 10/21/24.
//

import Foundation
import SwiftUI

class Vertex {
    var center : CGPoint
    let radius = CGFloat(5)
    var offset : CGSize = .zero
    
    init(center: CGPoint) {
        self.center = center
    }
    
    func getCenter() -> CGPoint {
        return CGPoint(x:center.x + offset.width,y:center.y + offset.height)
    }
    
    func drawIn(_ context: GraphicsContext) {
        let circle = Path(ellipseIn: CGRect(x: center.x + offset.width - radius, y: center.y + offset.height - radius, width: 2 * radius, height: 2 * radius))
        context.fill(circle,with: .color(.red))
    }
    
    func containsPoint(_ pt : CGPoint) -> Bool {
        let deltaX = (center.x - pt.x)
        let deltaY = (center.y - pt.y)
        let distSquared = deltaX*deltaX + deltaY*deltaY
        if distSquared < radius*radius {
            return true
        } else {
            return false
        }
    }
    
    func setOffset(_ offset : CGSize) {
        self.offset = offset
    }
    
    func endDrag() {
        center.x += offset.width
        center.y += offset.height
        offset = .zero
    }
}

class Edge {
    var origin : Vertex
    var dest : Vertex
    
    init(origin: Vertex, dest: Vertex) {
        self.origin = origin
        self.dest = dest
    }
    
    func drawIn(_ context: GraphicsContext) {
        let edge = Path() { path in
            path.move(to: origin.getCenter())
            path.addLine(to: dest.getCenter())
        }
        context.stroke(edge,with:.color(.green),lineWidth:2)
    }
}
